]> git.sesse.net Git - cubemap/blobdiff - server.h
Remove some leftover code.
[cubemap] / server.h
index 0f93b7f0b034caa1eb88a276f3b2aa40fa78fc83..fd309f41e34e1b97434f5af0071f9a47e4ad6690 100644 (file)
--- a/server.h
+++ b/server.h
@@ -3,20 +3,33 @@
 
 #include <stdint.h>
 #include <pthread.h>
+#include <sys/epoll.h>
 #include <string>
 #include <map>
+#include <vector>
 
-#define NUM_SERVERS 4
 #define BACKLOG_SIZE 1048576
 #define EPOLL_MAX_EVENTS 8192
 #define EPOLL_TIMEOUT_MS 20
 #define MAX_CLIENT_REQUEST 16384
 
+class ClientProto;
+class CubemapStateProto;
+class Stream;
+class StreamProto;
+
 struct Client {
+       Client() {}
+       Client(int sock);
+
+       // Serialization/deserialization.
+       Client(const ClientProto &serialized, Stream *stream);
+       ClientProto serialize() const;
+
        // The file descriptor associated with this socket.
        int sock;
 
-       enum State { READING_REQUEST, SENDING_HEADER, SENDING_DATA };
+       enum State { READING_REQUEST, SENDING_HEADER, SENDING_DATA, SENDING_ERROR };
        State state;
 
        // The HTTP request, as sent by the client. If we are in READING_REQUEST,
@@ -26,34 +39,49 @@ struct Client {
        // What stream we're connecting to; parsed from <request>.
        // Not relevant for READING_REQUEST.
        std::string stream_id;
+       Stream *stream;
 
        // The header we want to send. This is nominally a copy of Stream::header,
        // but since that might change on reconnects etc., we keep a local copy here.
-       // Only relevant for SENDING_HEADER; blank otherwise.
-       std::string header;
+       // Only relevant for SENDING_HEADER or SENDING_ERROR; blank otherwise.
+       std::string header_or_error;
 
-       // Number of bytes we've sent of the header. Only relevant for SENDING_HEADER.
-       size_t header_bytes_sent;
+       // Number of bytes we've sent of the header. Only relevant for SENDING_HEADER
+       // or SENDING_ERROR.
+       size_t header_or_error_bytes_sent;
 
        // Number of bytes we've sent of data. Only relevant for SENDING_DATA.
        size_t bytes_sent;
 };
 
 struct Stream {
+       Stream(const std::string &stream_id);
+       ~Stream();
+
+       // Serialization/deserialization.
+       Stream(const StreamProto &serialized);
+       StreamProto serialize() const;
+
+       std::string stream_id;
+
        // The HTTP response header, plus the video stream header (if any).
        std::string header;
 
        // The stream data itself, stored in a circular buffer.
-       char data[BACKLOG_SIZE];
+       char *data;
 
        // How many bytes <data> contains. Can very well be larger than BACKLOG_SIZE,
        // since the buffer wraps.
        size_t data_size;
+
+private:
+       Stream(const Stream& other);
 };
 
 class Server {
 public:
        Server();
+       ~Server();
 
        // Start a new thread that handles clients.
        void run();
@@ -61,8 +89,14 @@ public:
        // Stop the thread.
        void stop();
 
+       CubemapStateProto serialize() const;
+
        void add_client(int sock);
+       void add_client_from_serialized(const ClientProto &client);
+
        void add_stream(const std::string &stream_id);
+       void add_stream_from_serialized(const StreamProto &stream);
+
        void set_header(const std::string &stream_id, const std::string &header);
        void add_data(const std::string &stream_id, const char *data, size_t bytes);
 
@@ -76,7 +110,7 @@ private:
        bool should_stop;       
 
        // Map from stream ID to stream.
-       std::map<std::string, Stream> streams;
+       std::map<std::string, Stream *> streams;
 
        // Map from file descriptor to client.
        std::map<int, Client> clients;
@@ -88,7 +122,7 @@ private:
        // Clients that are in SENDING_DATA, but that we don't listen on,
        // because we currently don't have any data for them.
        // See put_client_to_sleep() and wake_up_all_clients().
-       std::vector<int> sleeping_clients;
+       std::vector<Client *> sleeping_clients;
 
        // Recover the this pointer, and call do_work().
        static void *do_work_thunk(void *arg);
@@ -96,14 +130,34 @@ private:
        // The actual worker thread.
        void do_work();
 
+       // Process a client; read and write data as far as we can.
+       // After this call, one of these four is true:
+       //
+       //  1. The socket is closed, and the client deleted.
+       //  2. We are still waiting for more data from the client.
+       //  3. We've sent all the data we have to the client,
+       //     and put it in <sleeping_clients>.
+       //  4. The socket buffer is full (which means we still have
+       //     data outstanding).
+       //
+       // For #2, we listen for EPOLLIN events. For #3 and #4, we listen
+       // for EPOLLOUT in edge-triggered mode; it will never fire for #3,
+       // but it's cheaper than taking it in and out all the time.
        void process_client(Client *client);
 
        // Close a given client socket, and clean up after it.
        void close_client(Client *client);
 
-       // Parse the HTTP request, construct the header, and set the client into
+       // Parse the HTTP request. Returns a HTTP status code (200/400/404).
+       int parse_request(Client *client);
+
+       // Construct the HTTP header, and set the client into
        // the SENDING_HEADER state.
-       void parse_request(Client *client);
+       void construct_header(Client *client);
+
+       // Construct a generic error with the given line, and set the client into
+       // the SENDING_ERROR state.
+       void construct_error(Client *client, int error_code);
 
        // Put client to sleep, since there is no more data for it; we will on
        // longer listen on POLLOUT until we get more data. Also, it will be put
@@ -112,6 +166,9 @@ private:
 
        // We have more data, so mark all clients that are sleeping as ready to go.
        void wake_up_all_clients();
+
+       // TODO: This function should probably die.
+       Stream *find_stream(const std::string &stream_id);
 };
 
 #endif  // !defined(_SERVER_H)